# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .json import (
    JsonResource,
    AsyncJsonResource,
    JsonResourceWithRawResponse,
    AsyncJsonResourceWithRawResponse,
    JsonResourceWithStreamingResponse,
    AsyncJsonResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["RecordingsResource", "AsyncRecordingsResource"]


class RecordingsResource(SyncAPIResource):
    @cached_property
    def json(self) -> JsonResource:
        return JsonResource(self._client)

    @cached_property
    def with_raw_response(self) -> RecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RecordingsResourceWithStreamingResponse(self)


class AsyncRecordingsResource(AsyncAPIResource):
    @cached_property
    def json(self) -> AsyncJsonResource:
        return AsyncJsonResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRecordingsResourceWithStreamingResponse(self)


class RecordingsResourceWithRawResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

    @cached_property
    def json(self) -> JsonResourceWithRawResponse:
        return JsonResourceWithRawResponse(self._recordings.json)


class AsyncRecordingsResourceWithRawResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

    @cached_property
    def json(self) -> AsyncJsonResourceWithRawResponse:
        return AsyncJsonResourceWithRawResponse(self._recordings.json)


class RecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

    @cached_property
    def json(self) -> JsonResourceWithStreamingResponse:
        return JsonResourceWithStreamingResponse(self._recordings.json)


class AsyncRecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

    @cached_property
    def json(self) -> AsyncJsonResourceWithStreamingResponse:
        return AsyncJsonResourceWithStreamingResponse(self._recordings.json)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .standard_pagination_meta import StandardPaginationMeta
from .cdr_usage_report_response_legacy import CdrUsageReportResponseLegacy

__all__ = ["VoiceListResponse"]


class VoiceListResponse(BaseModel):
    data: Optional[List[CdrUsageReportResponseLegacy]] = None

    meta: Optional[StandardPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SimCardDataUsageNotification", "Threshold"]


class Threshold(BaseModel):
    amount: Optional[str] = None

    unit: Optional[Literal["MB", "GB"]] = None


class SimCardDataUsageNotification(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    record_type: Optional[str] = None

    sim_card_id: Optional[str] = None
    """The identification UUID of the related SIM card resource."""

    threshold: Optional[Threshold] = None
    """Data usage threshold that will trigger the notification."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .entity_type import EntityType
from .brand_identity_status import BrandIdentityStatus

__all__ = ["BrandListResponse", "Record"]


class Record(BaseModel):
    assigned_campaings_count: Optional[int] = FieldInfo(alias="assignedCampaingsCount", default=None)
    """Number of campaigns associated with the brand"""

    brand_id: Optional[str] = FieldInfo(alias="brandId", default=None)
    """Unique identifier assigned to the brand."""

    company_name: Optional[str] = FieldInfo(alias="companyName", default=None)
    """(Required for Non-profit/private/public) Legal company name."""

    created_at: Optional[str] = FieldInfo(alias="createdAt", default=None)
    """Date and time that the brand was created at."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)
    """Display or marketing name of the brand."""

    email: Optional[str] = None
    """Valid email address of brand support contact."""

    entity_type: Optional[EntityType] = FieldInfo(alias="entityType", default=None)
    """Entity type behind the brand. This is the form of business establishment."""

    failure_reasons: Optional[str] = FieldInfo(alias="failureReasons", default=None)
    """Failure reasons for brand"""

    identity_status: Optional[BrandIdentityStatus] = FieldInfo(alias="identityStatus", default=None)
    """The verification status of an active brand"""

    status: Optional[Literal["OK", "REGISTRATION_PENDING", "REGISTRATION_FAILED"]] = None
    """Status of the brand"""

    tcr_brand_id: Optional[str] = FieldInfo(alias="tcrBrandId", default=None)
    """Unique identifier assigned to the brand by the registry."""

    updated_at: Optional[str] = FieldInfo(alias="updatedAt", default=None)
    """Date and time that the brand was last updated at."""

    website: Optional[str] = None
    """Brand website URL."""


class BrandListResponse(BaseModel):
    page: Optional[int] = None

    records: Optional[List[Record]] = None

    total_records: Optional[int] = FieldInfo(alias="totalRecords", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .standard_pagination_meta import StandardPaginationMeta
from .mdr_usage_report_response_legacy import MdrUsageReportResponseLegacy

__all__ = ["MessagingListResponse"]


class MessagingListResponse(BaseModel):
    data: Optional[List[MdrUsageReportResponseLegacy]] = None

    meta: Optional[StandardPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ActionAnswerResponse", "Data"]


class Data(BaseModel):
    recording_id: Optional[str] = None
    """The ID of the recording.

    Only present when the record parameter is set to record-from-answer.
    """

    result: Optional[str] = None


class ActionAnswerResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import CallEventListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCallEvents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        call_event = client.call_events.list()
        assert_matches_type(CallEventListResponse, call_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        call_event = client.call_events.list(
            filter={
                "application_name": {"contains": "contains"},
                "application_session_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "connection_id": "connection_id",
                "failed": False,
                "from": "+12025550142",
                "leg_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "name": "name",
                "occurred_at": {
                    "eq": "2019-03-29T11:10:00Z",
                    "gt": "2019-03-29T11:10:00Z",
                    "gte": "2019-03-29T11:10:00Z",
                    "lt": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "outbound_outbound_voice_profile_id": "outbound.outbound_voice_profile_id",
                "product": "texml",
                "status": "init",
                "to": "+12025550142",
                "type": "webhook",
            },
            page={
                "after": "after",
                "before": "before",
                "limit": 1,
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(CallEventListResponse, call_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.call_events.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_event = response.parse()
        assert_matches_type(CallEventListResponse, call_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.call_events.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_event = response.parse()
            assert_matches_type(CallEventListResponse, call_event, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCallEvents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        call_event = await async_client.call_events.list()
        assert_matches_type(CallEventListResponse, call_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        call_event = await async_client.call_events.list(
            filter={
                "application_name": {"contains": "contains"},
                "application_session_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "connection_id": "connection_id",
                "failed": False,
                "from": "+12025550142",
                "leg_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "name": "name",
                "occurred_at": {
                    "eq": "2019-03-29T11:10:00Z",
                    "gt": "2019-03-29T11:10:00Z",
                    "gte": "2019-03-29T11:10:00Z",
                    "lt": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "outbound_outbound_voice_profile_id": "outbound.outbound_voice_profile_id",
                "product": "texml",
                "status": "init",
                "to": "+12025550142",
                "type": "webhook",
            },
            page={
                "after": "after",
                "before": "before",
                "limit": 1,
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(CallEventListResponse, call_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.call_events.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_event = await response.parse()
        assert_matches_type(CallEventListResponse, call_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.call_events.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_event = await response.parse()
            assert_matches_type(CallEventListResponse, call_event, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .record import Record
from .._models import BaseModel
from .interface import Interface
from .pagination_meta import PaginationMeta

__all__ = ["NetworkListInterfacesResponse", "Data", "DataRegion"]


class DataRegion(BaseModel):
    code: Optional[str] = None
    """Region code of the interface."""

    name: Optional[str] = None
    """Region name of the interface."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class Data(Record, Interface):
    record_type: Optional[str] = None  # type: ignore
    """Identifies the type of the resource."""

    region: Optional[DataRegion] = None

    region_code: Optional[str] = None
    """The region interface is deployed to."""

    type: Optional[str] = None
    """Identifies the type of the interface."""


class NetworkListInterfacesResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime

from ...._models import BaseModel
from .version_config import VersionConfig

__all__ = ["CanaryDeployResponse"]


class CanaryDeployResponse(BaseModel):
    assistant_id: str

    created_at: datetime

    updated_at: datetime

    versions: List[VersionConfig]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CallForwarding"]


class CallForwarding(BaseModel):
    call_forwarding_enabled: Optional[bool] = None
    """
    Indicates if call forwarding will be enabled for this number if forwards_to and
    forwarding_type are filled in. Defaults to true for backwards compatibility with
    APIV1 use of numbers endpoints.
    """

    forwarding_type: Optional[Literal["always", "on-failure"]] = None
    """Call forwarding type. 'forwards_to' must be set for this to have an effect."""

    forwards_to: Optional[str] = None
    """The phone number to which inbound calls to this number are forwarded.

    Inbound calls will not be forwarded if this field is left blank. If set, must be
    a +E.164-formatted phone number.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .fax import Fax
from .._models import BaseModel

__all__ = ["FaxListResponse"]


class FaxListResponse(BaseModel):
    data: Optional[List[Fax]] = None

    meta: Optional[object] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .record import Record
from .._models import BaseModel

__all__ = ["NetworkCreateResponse", "Data"]


class Data(Record):
    name: Optional[str] = None
    """A user specified name for the network."""

    record_type: Optional[str] = None  # type: ignore
    """Identifies the type of the resource."""


class NetworkCreateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import wireguard_peer_list_params, wireguard_peer_create_params, wireguard_peer_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.wireguard_peer_list_response import WireguardPeerListResponse
from ..types.wireguard_peer_create_response import WireguardPeerCreateResponse
from ..types.wireguard_peer_delete_response import WireguardPeerDeleteResponse
from ..types.wireguard_peer_update_response import WireguardPeerUpdateResponse
from ..types.wireguard_peer_retrieve_response import WireguardPeerRetrieveResponse

__all__ = ["WireguardPeersResource", "AsyncWireguardPeersResource"]


class WireguardPeersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WireguardPeersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return WireguardPeersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WireguardPeersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return WireguardPeersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        wireguard_interface_id: str,
        public_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerCreateResponse:
        """Create a new WireGuard Peer.

        Current limitation of 5 peers per interface can be
        created.

        Args:
          wireguard_interface_id: The id of the wireguard interface associated with the peer.

          public_key: The WireGuard `PublicKey`.<br /><br />If you do not provide a Public Key, a new
              Public and Private key pair will be generated for you.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/wireguard_peers",
            body=maybe_transform(
                {
                    "wireguard_interface_id": wireguard_interface_id,
                    "public_key": public_key,
                },
                wireguard_peer_create_params.WireguardPeerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerRetrieveResponse:
        """
        Retrieve the WireGuard peer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/wireguard_peers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        public_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerUpdateResponse:
        """
        Update the WireGuard peer.

        Args:
          public_key: The WireGuard `PublicKey`.<br /><br />If you do not provide a Public Key, a new
              Public and Private key pair will be generated for you.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/wireguard_peers/{id}",
            body=maybe_transform({"public_key": public_key}, wireguard_peer_update_params.WireguardPeerUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerUpdateResponse,
        )

    def list(
        self,
        *,
        filter: wireguard_peer_list_params.Filter | Omit = omit,
        page: wireguard_peer_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerListResponse:
        """
        List all WireGuard peers.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[wireguard_interface_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/wireguard_peers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    wireguard_peer_list_params.WireguardPeerListParams,
                ),
            ),
            cast_to=WireguardPeerListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerDeleteResponse:
        """
        Delete the WireGuard peer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/wireguard_peers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerDeleteResponse,
        )

    def retrieve_config(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Retrieve Wireguard config template for Peer

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._get(
            f"/wireguard_peers/{id}/config",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AsyncWireguardPeersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWireguardPeersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWireguardPeersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWireguardPeersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncWireguardPeersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        wireguard_interface_id: str,
        public_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerCreateResponse:
        """Create a new WireGuard Peer.

        Current limitation of 5 peers per interface can be
        created.

        Args:
          wireguard_interface_id: The id of the wireguard interface associated with the peer.

          public_key: The WireGuard `PublicKey`.<br /><br />If you do not provide a Public Key, a new
              Public and Private key pair will be generated for you.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/wireguard_peers",
            body=await async_maybe_transform(
                {
                    "wireguard_interface_id": wireguard_interface_id,
                    "public_key": public_key,
                },
                wireguard_peer_create_params.WireguardPeerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerRetrieveResponse:
        """
        Retrieve the WireGuard peer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/wireguard_peers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        public_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerUpdateResponse:
        """
        Update the WireGuard peer.

        Args:
          public_key: The WireGuard `PublicKey`.<br /><br />If you do not provide a Public Key, a new
              Public and Private key pair will be generated for you.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/wireguard_peers/{id}",
            body=await async_maybe_transform(
                {"public_key": public_key}, wireguard_peer_update_params.WireguardPeerUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: wireguard_peer_list_params.Filter | Omit = omit,
        page: wireguard_peer_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerListResponse:
        """
        List all WireGuard peers.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[wireguard_interface_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/wireguard_peers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    wireguard_peer_list_params.WireguardPeerListParams,
                ),
            ),
            cast_to=WireguardPeerListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WireguardPeerDeleteResponse:
        """
        Delete the WireGuard peer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/wireguard_peers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WireguardPeerDeleteResponse,
        )

    async def retrieve_config(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Retrieve Wireguard config template for Peer

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._get(
            f"/wireguard_peers/{id}/config",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class WireguardPeersResourceWithRawResponse:
    def __init__(self, wireguard_peers: WireguardPeersResource) -> None:
        self._wireguard_peers = wireguard_peers

        self.create = to_raw_response_wrapper(
            wireguard_peers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            wireguard_peers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            wireguard_peers.update,
        )
        self.list = to_raw_response_wrapper(
            wireguard_peers.list,
        )
        self.delete = to_raw_response_wrapper(
            wireguard_peers.delete,
        )
        self.retrieve_config = to_raw_response_wrapper(
            wireguard_peers.retrieve_config,
        )


class AsyncWireguardPeersResourceWithRawResponse:
    def __init__(self, wireguard_peers: AsyncWireguardPeersResource) -> None:
        self._wireguard_peers = wireguard_peers

        self.create = async_to_raw_response_wrapper(
            wireguard_peers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            wireguard_peers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            wireguard_peers.update,
        )
        self.list = async_to_raw_response_wrapper(
            wireguard_peers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            wireguard_peers.delete,
        )
        self.retrieve_config = async_to_raw_response_wrapper(
            wireguard_peers.retrieve_config,
        )


class WireguardPeersResourceWithStreamingResponse:
    def __init__(self, wireguard_peers: WireguardPeersResource) -> None:
        self._wireguard_peers = wireguard_peers

        self.create = to_streamed_response_wrapper(
            wireguard_peers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            wireguard_peers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            wireguard_peers.update,
        )
        self.list = to_streamed_response_wrapper(
            wireguard_peers.list,
        )
        self.delete = to_streamed_response_wrapper(
            wireguard_peers.delete,
        )
        self.retrieve_config = to_streamed_response_wrapper(
            wireguard_peers.retrieve_config,
        )


class AsyncWireguardPeersResourceWithStreamingResponse:
    def __init__(self, wireguard_peers: AsyncWireguardPeersResource) -> None:
        self._wireguard_peers = wireguard_peers

        self.create = async_to_streamed_response_wrapper(
            wireguard_peers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            wireguard_peers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            wireguard_peers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            wireguard_peers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            wireguard_peers.delete,
        )
        self.retrieve_config = async_to_streamed_response_wrapper(
            wireguard_peers.retrieve_config,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["MobilePushCredentialListParams", "Filter", "Page"]


class MobilePushCredentialListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[type], filter[alias]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class Filter(TypedDict, total=False):
    alias: str
    """Unique mobile push credential alias"""

    type: Literal["ios", "android"]
    """type of mobile push credentials"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load."""

    size: int
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .sip_header import SipHeader
from .custom_sip_header import CustomSipHeader

__all__ = [
    "CallHangupWebhookEvent",
    "Data",
    "DataPayload",
    "DataPayloadCallQualityStats",
    "DataPayloadCallQualityStatsInbound",
    "DataPayloadCallQualityStatsOutbound",
]


class DataPayloadCallQualityStatsInbound(BaseModel):
    jitter_max_variance: Optional[str] = None
    """Maximum jitter variance for inbound audio."""

    jitter_packet_count: Optional[str] = None
    """Number of packets used for jitter calculation on inbound audio."""

    mos: Optional[str] = None
    """Mean Opinion Score (MOS) for inbound audio quality."""

    packet_count: Optional[str] = None
    """Total number of inbound audio packets."""

    skip_packet_count: Optional[str] = None
    """Number of skipped inbound packets (packet loss)."""


class DataPayloadCallQualityStatsOutbound(BaseModel):
    packet_count: Optional[str] = None
    """Total number of outbound audio packets."""

    skip_packet_count: Optional[str] = None
    """Number of skipped outbound packets (packet loss)."""


class DataPayloadCallQualityStats(BaseModel):
    inbound: Optional[DataPayloadCallQualityStatsInbound] = None
    """Inbound call quality statistics."""

    outbound: Optional[DataPayloadCallQualityStatsOutbound] = None
    """Outbound call quality statistics."""


class DataPayload(BaseModel):
    call_control_id: Optional[str] = None
    """Call ID used to issue commands via Call Control API."""

    call_leg_id: Optional[str] = None
    """ID that is unique to the call and can be used to correlate webhook events."""

    call_quality_stats: Optional[DataPayloadCallQualityStats] = None
    """Call quality statistics aggregated from the CHANNEL_HANGUP_COMPLETE event.

    Only includes metrics that are available (filters out nil values). Returns nil
    if no metrics are available.
    """

    call_session_id: Optional[str] = None
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call.
    """

    client_state: Optional[str] = None
    """State received from a command."""

    connection_id: Optional[str] = None
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    custom_headers: Optional[List[CustomSipHeader]] = None
    """Custom headers set on answer command"""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """Number or SIP URI placing the call."""

    hangup_cause: Optional[
        Literal[
            "call_rejected",
            "normal_clearing",
            "originator_cancel",
            "timeout",
            "time_limit",
            "user_busy",
            "not_found",
            "unspecified",
        ]
    ] = None
    """
    The reason the call was ended (`call_rejected`, `normal_clearing`,
    `originator_cancel`, `timeout`, `time_limit`, `user_busy`, `not_found` or
    `unspecified`).
    """

    hangup_source: Optional[Literal["caller", "callee", "unknown"]] = None
    """The party who ended the call (`callee`, `caller`, `unknown`)."""

    sip_hangup_cause: Optional[str] = None
    """The reason the call was ended (SIP response code).

    If the SIP response is unavailable (in inbound calls for example) this is set to
    `unspecified`.
    """

    sip_headers: Optional[List[SipHeader]] = None
    """User-to-User and Diversion headers from sip invite."""

    start_time: Optional[datetime] = None
    """ISO 8601 datetime of when the call started."""

    state: Optional[Literal["hangup"]] = None
    """State received from a command."""

    tags: Optional[List[str]] = None
    """Array of tags associated to number."""

    to: Optional[str] = None
    """Destination number or SIP URI of the call."""


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["call.hangup"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class CallHangupWebhookEvent(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.sim_card_groups import (
    action_list_params,
    action_set_wireless_blocklist_params,
    action_set_private_wireless_gateway_params,
)
from ...types.sim_card_groups.action_list_response import ActionListResponse
from ...types.sim_card_groups.action_retrieve_response import ActionRetrieveResponse
from ...types.sim_card_groups.action_set_wireless_blocklist_response import ActionSetWirelessBlocklistResponse
from ...types.sim_card_groups.action_remove_wireless_blocklist_response import ActionRemoveWirelessBlocklistResponse
from ...types.sim_card_groups.action_set_private_wireless_gateway_response import (
    ActionSetPrivateWirelessGatewayResponse,
)
from ...types.sim_card_groups.action_remove_private_wireless_gateway_response import (
    ActionRemovePrivateWirelessGatewayResponse,
)

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRetrieveResponse:
        """
        This API allows fetching detailed information about a SIM card group action
        resource to make follow-ups in an existing asynchronous operation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_card_group_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter_sim_card_group_id: str | Omit = omit,
        filter_status: Literal["in-progress", "completed", "failed"] | Omit = omit,
        filter_type: Literal[
            "set_private_wireless_gateway",
            "remove_private_wireless_gateway",
            "set_wireless_blocklist",
            "remove_wireless_blocklist",
        ]
        | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionListResponse:
        """This API allows listing a paginated collection a SIM card group actions.

        It
        allows to explore a collection of existing asynchronous operation using specific
        filters.

        Args:
          filter_sim_card_group_id: A valid SIM card group ID.

          filter_status: Filter by a specific status of the resource's lifecycle.

          filter_type: Filter by action type.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/sim_card_group_actions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_sim_card_group_id": filter_sim_card_group_id,
                        "filter_status": filter_status,
                        "filter_type": filter_type,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    action_list_params.ActionListParams,
                ),
            ),
            cast_to=ActionListResponse,
        )

    def remove_private_wireless_gateway(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRemovePrivateWirelessGatewayResponse:
        """
        This action will asynchronously remove an existing Private Wireless Gateway
        definition from a SIM card group. Completing this operation defines that all SIM
        cards in the SIM card group will get their traffic handled by Telnyx's default
        mobile network configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_card_groups/{id}/actions/remove_private_wireless_gateway",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRemovePrivateWirelessGatewayResponse,
        )

    def remove_wireless_blocklist(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRemoveWirelessBlocklistResponse:
        """
        This action will asynchronously remove an existing Wireless Blocklist to all the
        SIMs in the SIM card group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_card_groups/{id}/actions/remove_wireless_blocklist",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRemoveWirelessBlocklistResponse,
        )

    def set_private_wireless_gateway(
        self,
        id: str,
        *,
        private_wireless_gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetPrivateWirelessGatewayResponse:
        """
        This action will asynchronously assign a provisioned Private Wireless Gateway to
        the SIM card group. Completing this operation defines that all SIM cards in the
        SIM card group will get their traffic controlled by the associated Private
        Wireless Gateway. This operation will also imply that new SIM cards assigned to
        a group will inherit its network definitions. If it's moved to a different group
        that doesn't have a Private Wireless Gateway, it'll use Telnyx's default mobile
        network configuration.

        Args:
          private_wireless_gateway_id: The identification of the related Private Wireless Gateway resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_card_groups/{id}/actions/set_private_wireless_gateway",
            body=maybe_transform(
                {"private_wireless_gateway_id": private_wireless_gateway_id},
                action_set_private_wireless_gateway_params.ActionSetPrivateWirelessGatewayParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSetPrivateWirelessGatewayResponse,
        )

    def set_wireless_blocklist(
        self,
        id: str,
        *,
        wireless_blocklist_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetWirelessBlocklistResponse:
        """
        This action will asynchronously assign a Wireless Blocklist to all the SIMs in
        the SIM card group.

        Args:
          wireless_blocklist_id: The identification of the related Wireless Blocklist resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_card_groups/{id}/actions/set_wireless_blocklist",
            body=maybe_transform(
                {"wireless_blocklist_id": wireless_blocklist_id},
                action_set_wireless_blocklist_params.ActionSetWirelessBlocklistParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSetWirelessBlocklistResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRetrieveResponse:
        """
        This API allows fetching detailed information about a SIM card group action
        resource to make follow-ups in an existing asynchronous operation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_card_group_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter_sim_card_group_id: str | Omit = omit,
        filter_status: Literal["in-progress", "completed", "failed"] | Omit = omit,
        filter_type: Literal[
            "set_private_wireless_gateway",
            "remove_private_wireless_gateway",
            "set_wireless_blocklist",
            "remove_wireless_blocklist",
        ]
        | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionListResponse:
        """This API allows listing a paginated collection a SIM card group actions.

        It
        allows to explore a collection of existing asynchronous operation using specific
        filters.

        Args:
          filter_sim_card_group_id: A valid SIM card group ID.

          filter_status: Filter by a specific status of the resource's lifecycle.

          filter_type: Filter by action type.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/sim_card_group_actions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter_sim_card_group_id": filter_sim_card_group_id,
                        "filter_status": filter_status,
                        "filter_type": filter_type,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    action_list_params.ActionListParams,
                ),
            ),
            cast_to=ActionListResponse,
        )

    async def remove_private_wireless_gateway(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRemovePrivateWirelessGatewayResponse:
        """
        This action will asynchronously remove an existing Private Wireless Gateway
        definition from a SIM card group. Completing this operation defines that all SIM
        cards in the SIM card group will get their traffic handled by Telnyx's default
        mobile network configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_card_groups/{id}/actions/remove_private_wireless_gateway",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRemovePrivateWirelessGatewayResponse,
        )

    async def remove_wireless_blocklist(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRemoveWirelessBlocklistResponse:
        """
        This action will asynchronously remove an existing Wireless Blocklist to all the
        SIMs in the SIM card group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_card_groups/{id}/actions/remove_wireless_blocklist",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRemoveWirelessBlocklistResponse,
        )

    async def set_private_wireless_gateway(
        self,
        id: str,
        *,
        private_wireless_gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetPrivateWirelessGatewayResponse:
        """
        This action will asynchronously assign a provisioned Private Wireless Gateway to
        the SIM card group. Completing this operation defines that all SIM cards in the
        SIM card group will get their traffic controlled by the associated Private
        Wireless Gateway. This operation will also imply that new SIM cards assigned to
        a group will inherit its network definitions. If it's moved to a different group
        that doesn't have a Private Wireless Gateway, it'll use Telnyx's default mobile
        network configuration.

        Args:
          private_wireless_gateway_id: The identification of the related Private Wireless Gateway resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_card_groups/{id}/actions/set_private_wireless_gateway",
            body=await async_maybe_transform(
                {"private_wireless_gateway_id": private_wireless_gateway_id},
                action_set_private_wireless_gateway_params.ActionSetPrivateWirelessGatewayParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSetPrivateWirelessGatewayResponse,
        )

    async def set_wireless_blocklist(
        self,
        id: str,
        *,
        wireless_blocklist_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetWirelessBlocklistResponse:
        """
        This action will asynchronously assign a Wireless Blocklist to all the SIMs in
        the SIM card group.

        Args:
          wireless_blocklist_id: The identification of the related Wireless Blocklist resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_card_groups/{id}/actions/set_wireless_blocklist",
            body=await async_maybe_transform(
                {"wireless_blocklist_id": wireless_blocklist_id},
                action_set_wireless_blocklist_params.ActionSetWirelessBlocklistParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSetWirelessBlocklistResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.retrieve = to_raw_response_wrapper(
            actions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            actions.list,
        )
        self.remove_private_wireless_gateway = to_raw_response_wrapper(
            actions.remove_private_wireless_gateway,
        )
        self.remove_wireless_blocklist = to_raw_response_wrapper(
            actions.remove_wireless_blocklist,
        )
        self.set_private_wireless_gateway = to_raw_response_wrapper(
            actions.set_private_wireless_gateway,
        )
        self.set_wireless_blocklist = to_raw_response_wrapper(
            actions.set_wireless_blocklist,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.retrieve = async_to_raw_response_wrapper(
            actions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            actions.list,
        )
        self.remove_private_wireless_gateway = async_to_raw_response_wrapper(
            actions.remove_private_wireless_gateway,
        )
        self.remove_wireless_blocklist = async_to_raw_response_wrapper(
            actions.remove_wireless_blocklist,
        )
        self.set_private_wireless_gateway = async_to_raw_response_wrapper(
            actions.set_private_wireless_gateway,
        )
        self.set_wireless_blocklist = async_to_raw_response_wrapper(
            actions.set_wireless_blocklist,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.retrieve = to_streamed_response_wrapper(
            actions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            actions.list,
        )
        self.remove_private_wireless_gateway = to_streamed_response_wrapper(
            actions.remove_private_wireless_gateway,
        )
        self.remove_wireless_blocklist = to_streamed_response_wrapper(
            actions.remove_wireless_blocklist,
        )
        self.set_private_wireless_gateway = to_streamed_response_wrapper(
            actions.set_private_wireless_gateway,
        )
        self.set_wireless_blocklist = to_streamed_response_wrapper(
            actions.set_wireless_blocklist,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.retrieve = async_to_streamed_response_wrapper(
            actions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            actions.list,
        )
        self.remove_private_wireless_gateway = async_to_streamed_response_wrapper(
            actions.remove_private_wireless_gateway,
        )
        self.remove_wireless_blocklist = async_to_streamed_response_wrapper(
            actions.remove_wireless_blocklist,
        )
        self.set_private_wireless_gateway = async_to_streamed_response_wrapper(
            actions.set_private_wireless_gateway,
        )
        self.set_wireless_blocklist = async_to_streamed_response_wrapper(
            actions.set_wireless_blocklist,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MessagingOptoutListParams", "CreatedAt", "Filter", "Page"]


class MessagingOptoutListParams(TypedDict, total=False):
    created_at: CreatedAt
    """Consolidated created_at parameter (deepObject style).

    Originally: created_at[gte], created_at[lte]
    """

    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[messaging_profile_id], filter[from]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """

    redaction_enabled: str
    """If receiving address (+E.164 formatted phone number) should be redacted"""


class CreatedAt(TypedDict, total=False):
    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter opt-outs created after this date (ISO-8601 format)"""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter opt-outs created before this date (ISO-8601 format)"""


_FilterReservedKeywords = TypedDict(
    "_FilterReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class Filter(_FilterReservedKeywords, total=False):
    messaging_profile_id: str
    """The ID of the messaging profile to retrieve opt-outs for"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .inference_embedding_webhook_tool_params_param import InferenceEmbeddingWebhookToolParamsParam

__all__ = ["WebhookToolParam"]


class WebhookToolParam(TypedDict, total=False):
    type: Required[Literal["webhook"]]

    webhook: Required[InferenceEmbeddingWebhookToolParamsParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ConferenceRetrieveConferencesParams"]


class ConferenceRetrieveConferencesParams(TypedDict, total=False):
    date_created: Annotated[str, PropertyInfo(alias="DateCreated")]
    """Filters conferences by the creation date.

    Expected format is YYYY-MM-DD. Also accepts inequality operators, e.g.
    DateCreated>=2023-05-22.
    """

    date_updated: Annotated[str, PropertyInfo(alias="DateUpdated")]
    """Filters conferences by the time they were last updated.

    Expected format is YYYY-MM-DD. Also accepts inequality operators, e.g.
    DateUpdated>=2023-05-22.
    """

    friendly_name: Annotated[str, PropertyInfo(alias="FriendlyName")]
    """Filters conferences by their friendly name."""

    page: Annotated[int, PropertyInfo(alias="Page")]
    """
    The number of the page to be displayed, zero-indexed, should be used in
    conjuction with PageToken.
    """

    page_size: Annotated[int, PropertyInfo(alias="PageSize")]
    """The number of records to be displayed on a page"""

    page_token: Annotated[str, PropertyInfo(alias="PageToken")]
    """Used to request the next page of results."""

    status: Annotated[Literal["init", "in-progress", "completed"], PropertyInfo(alias="Status")]
    """Filters conferences by status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from ..types import room_composition_list_params, room_composition_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.video_region_param import VideoRegionParam
from ..types.room_composition_list_response import RoomCompositionListResponse
from ..types.room_composition_create_response import RoomCompositionCreateResponse
from ..types.room_composition_retrieve_response import RoomCompositionRetrieveResponse

__all__ = ["RoomCompositionsResource", "AsyncRoomCompositionsResource"]


class RoomCompositionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RoomCompositionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RoomCompositionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RoomCompositionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RoomCompositionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        format: Optional[str] | Omit = omit,
        resolution: Optional[str] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        video_layout: Dict[str, VideoRegionParam] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCompositionCreateResponse:
        """
        Asynchronously create a room composition.

        Args:
          format: The desired format of the room composition.

          resolution: The desired resolution (width/height in pixels) of the resulting video of the
              room composition. Both width and height are required to be between 16 and 1280;
              and width _ height should not exceed 1280 _ 720

          session_id: id of the room session associated with the room composition.

          video_layout: Describes the video layout of the room composition in terms of regions.

          webhook_event_failover_url: The failover URL where webhooks related to this room composition will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this room composition will be sent. Must
              include a scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/room_compositions",
            body=maybe_transform(
                {
                    "format": format,
                    "resolution": resolution,
                    "session_id": session_id,
                    "video_layout": video_layout,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                room_composition_create_params.RoomCompositionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomCompositionCreateResponse,
        )

    def retrieve(
        self,
        room_composition_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCompositionRetrieveResponse:
        """
        View a room composition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_composition_id:
            raise ValueError(
                f"Expected a non-empty value for `room_composition_id` but received {room_composition_id!r}"
            )
        return self._get(
            f"/room_compositions/{room_composition_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomCompositionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: room_composition_list_params.Filter | Omit = omit,
        page: room_composition_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCompositionListResponse:
        """
        View a list of room compositions.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[session_id], filter[status]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/room_compositions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    room_composition_list_params.RoomCompositionListParams,
                ),
            ),
            cast_to=RoomCompositionListResponse,
        )

    def delete(
        self,
        room_composition_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Synchronously delete a room composition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_composition_id:
            raise ValueError(
                f"Expected a non-empty value for `room_composition_id` but received {room_composition_id!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/room_compositions/{room_composition_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncRoomCompositionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRoomCompositionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRoomCompositionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRoomCompositionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRoomCompositionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        format: Optional[str] | Omit = omit,
        resolution: Optional[str] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        video_layout: Dict[str, VideoRegionParam] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCompositionCreateResponse:
        """
        Asynchronously create a room composition.

        Args:
          format: The desired format of the room composition.

          resolution: The desired resolution (width/height in pixels) of the resulting video of the
              room composition. Both width and height are required to be between 16 and 1280;
              and width _ height should not exceed 1280 _ 720

          session_id: id of the room session associated with the room composition.

          video_layout: Describes the video layout of the room composition in terms of regions.

          webhook_event_failover_url: The failover URL where webhooks related to this room composition will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this room composition will be sent. Must
              include a scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/room_compositions",
            body=await async_maybe_transform(
                {
                    "format": format,
                    "resolution": resolution,
                    "session_id": session_id,
                    "video_layout": video_layout,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                room_composition_create_params.RoomCompositionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomCompositionCreateResponse,
        )

    async def retrieve(
        self,
        room_composition_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCompositionRetrieveResponse:
        """
        View a room composition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_composition_id:
            raise ValueError(
                f"Expected a non-empty value for `room_composition_id` but received {room_composition_id!r}"
            )
        return await self._get(
            f"/room_compositions/{room_composition_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomCompositionRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter: room_composition_list_params.Filter | Omit = omit,
        page: room_composition_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCompositionListResponse:
        """
        View a list of room compositions.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[session_id], filter[status]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/room_compositions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    room_composition_list_params.RoomCompositionListParams,
                ),
            ),
            cast_to=RoomCompositionListResponse,
        )

    async def delete(
        self,
        room_composition_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Synchronously delete a room composition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_composition_id:
            raise ValueError(
                f"Expected a non-empty value for `room_composition_id` but received {room_composition_id!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/room_compositions/{room_composition_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class RoomCompositionsResourceWithRawResponse:
    def __init__(self, room_compositions: RoomCompositionsResource) -> None:
        self._room_compositions = room_compositions

        self.create = to_raw_response_wrapper(
            room_compositions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            room_compositions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            room_compositions.list,
        )
        self.delete = to_raw_response_wrapper(
            room_compositions.delete,
        )


class AsyncRoomCompositionsResourceWithRawResponse:
    def __init__(self, room_compositions: AsyncRoomCompositionsResource) -> None:
        self._room_compositions = room_compositions

        self.create = async_to_raw_response_wrapper(
            room_compositions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            room_compositions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            room_compositions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            room_compositions.delete,
        )


class RoomCompositionsResourceWithStreamingResponse:
    def __init__(self, room_compositions: RoomCompositionsResource) -> None:
        self._room_compositions = room_compositions

        self.create = to_streamed_response_wrapper(
            room_compositions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            room_compositions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            room_compositions.list,
        )
        self.delete = to_streamed_response_wrapper(
            room_compositions.delete,
        )


class AsyncRoomCompositionsResourceWithStreamingResponse:
    def __init__(self, room_compositions: AsyncRoomCompositionsResource) -> None:
        self._room_compositions = room_compositions

        self.create = async_to_streamed_response_wrapper(
            room_compositions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            room_compositions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            room_compositions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            room_compositions.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TranscriptionSettingsParam"]


class TranscriptionSettingsParam(TypedDict, total=False):
    language: str
    """The language of the audio to be transcribed.

    This is only applicable for `openai/whisper-large-v3-turbo` model. If not set,
    of if set to `auto`, the model will automatically detect the language. For the
    full list of supported languages, see the
    [whisper tokenizer](https://github.com/openai/whisper/blob/main/whisper/tokenizer.py).
    """

    model: str
    """The speech to text model to be used by the voice assistant.

    - `distil-whisper/distil-large-v2` is lower latency but English-only.
    - `openai/whisper-large-v3-turbo` is multi-lingual with automatic language
      detection but slightly higher latency.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...types import room_list_params, room_create_params, room_update_params, room_retrieve_params
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .sessions.sessions import (
    SessionsResource,
    AsyncSessionsResource,
    SessionsResourceWithRawResponse,
    AsyncSessionsResourceWithRawResponse,
    SessionsResourceWithStreamingResponse,
    AsyncSessionsResourceWithStreamingResponse,
)
from ...types.room_list_response import RoomListResponse
from ...types.room_create_response import RoomCreateResponse
from ...types.room_update_response import RoomUpdateResponse
from ...types.room_retrieve_response import RoomRetrieveResponse

__all__ = ["RoomsResource", "AsyncRoomsResource"]


class RoomsResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def sessions(self) -> SessionsResource:
        return SessionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RoomsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RoomsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RoomsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RoomsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        enable_recording: bool | Omit = omit,
        max_participants: int | Omit = omit,
        unique_name: str | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCreateResponse:
        """
        Synchronously create a Room.

        Args:
          enable_recording: Enable or disable recording for that room.

          max_participants: The maximum amount of participants allowed in a room. If new participants try to
              join after that limit is reached, their request will be rejected.

          unique_name: The unique (within the Telnyx account scope) name of the room.

          webhook_event_failover_url: The failover URL where webhooks related to this room will be sent if sending to
              the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this room will be sent. Must include a scheme,
              such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/rooms",
            body=maybe_transform(
                {
                    "enable_recording": enable_recording,
                    "max_participants": max_participants,
                    "unique_name": unique_name,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                room_create_params.RoomCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomCreateResponse,
        )

    def retrieve(
        self,
        room_id: str,
        *,
        include_sessions: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomRetrieveResponse:
        """
        View a room.

        Args:
          include_sessions: To decide if room sessions should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return self._get(
            f"/rooms/{room_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include_sessions": include_sessions}, room_retrieve_params.RoomRetrieveParams),
            ),
            cast_to=RoomRetrieveResponse,
        )

    def update(
        self,
        room_id: str,
        *,
        enable_recording: bool | Omit = omit,
        max_participants: int | Omit = omit,
        unique_name: str | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomUpdateResponse:
        """
        Synchronously update a Room.

        Args:
          enable_recording: Enable or disable recording for that room.

          max_participants: The maximum amount of participants allowed in a room. If new participants try to
              join after that limit is reached, their request will be rejected.

          unique_name: The unique (within the Telnyx account scope) name of the room.

          webhook_event_failover_url: The failover URL where webhooks related to this room will be sent if sending to
              the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this room will be sent. Must include a scheme,
              such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return self._patch(
            f"/rooms/{room_id}",
            body=maybe_transform(
                {
                    "enable_recording": enable_recording,
                    "max_participants": max_participants,
                    "unique_name": unique_name,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                room_update_params.RoomUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomUpdateResponse,
        )

    def list(
        self,
        *,
        filter: room_list_params.Filter | Omit = omit,
        include_sessions: bool | Omit = omit,
        page: room_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomListResponse:
        """
        View a list of rooms.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte], filter[unique_name]

          include_sessions: To decide if room sessions should be included in the response.

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/rooms",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_sessions": include_sessions,
                        "page": page,
                    },
                    room_list_params.RoomListParams,
                ),
            ),
            cast_to=RoomListResponse,
        )

    def delete(
        self,
        room_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Synchronously delete a Room.

        Participants from that room will be kicked out,
        they won't be able to join that room anymore, and you won't be charged anymore
        for that room.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/rooms/{room_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncRoomsResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def sessions(self) -> AsyncSessionsResource:
        return AsyncSessionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRoomsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRoomsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRoomsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRoomsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        enable_recording: bool | Omit = omit,
        max_participants: int | Omit = omit,
        unique_name: str | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomCreateResponse:
        """
        Synchronously create a Room.

        Args:
          enable_recording: Enable or disable recording for that room.

          max_participants: The maximum amount of participants allowed in a room. If new participants try to
              join after that limit is reached, their request will be rejected.

          unique_name: The unique (within the Telnyx account scope) name of the room.

          webhook_event_failover_url: The failover URL where webhooks related to this room will be sent if sending to
              the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this room will be sent. Must include a scheme,
              such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/rooms",
            body=await async_maybe_transform(
                {
                    "enable_recording": enable_recording,
                    "max_participants": max_participants,
                    "unique_name": unique_name,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                room_create_params.RoomCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomCreateResponse,
        )

    async def retrieve(
        self,
        room_id: str,
        *,
        include_sessions: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomRetrieveResponse:
        """
        View a room.

        Args:
          include_sessions: To decide if room sessions should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return await self._get(
            f"/rooms/{room_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_sessions": include_sessions}, room_retrieve_params.RoomRetrieveParams
                ),
            ),
            cast_to=RoomRetrieveResponse,
        )

    async def update(
        self,
        room_id: str,
        *,
        enable_recording: bool | Omit = omit,
        max_participants: int | Omit = omit,
        unique_name: str | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomUpdateResponse:
        """
        Synchronously update a Room.

        Args:
          enable_recording: Enable or disable recording for that room.

          max_participants: The maximum amount of participants allowed in a room. If new participants try to
              join after that limit is reached, their request will be rejected.

          unique_name: The unique (within the Telnyx account scope) name of the room.

          webhook_event_failover_url: The failover URL where webhooks related to this room will be sent if sending to
              the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this room will be sent. Must include a scheme,
              such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return await self._patch(
            f"/rooms/{room_id}",
            body=await async_maybe_transform(
                {
                    "enable_recording": enable_recording,
                    "max_participants": max_participants,
                    "unique_name": unique_name,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                room_update_params.RoomUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: room_list_params.Filter | Omit = omit,
        include_sessions: bool | Omit = omit,
        page: room_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomListResponse:
        """
        View a list of rooms.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte], filter[unique_name]

          include_sessions: To decide if room sessions should be included in the response.

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/rooms",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "include_sessions": include_sessions,
                        "page": page,
                    },
                    room_list_params.RoomListParams,
                ),
            ),
            cast_to=RoomListResponse,
        )

    async def delete(
        self,
        room_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Synchronously delete a Room.

        Participants from that room will be kicked out,
        they won't be able to join that room anymore, and you won't be charged anymore
        for that room.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/rooms/{room_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class RoomsResourceWithRawResponse:
    def __init__(self, rooms: RoomsResource) -> None:
        self._rooms = rooms

        self.create = to_raw_response_wrapper(
            rooms.create,
        )
        self.retrieve = to_raw_response_wrapper(
            rooms.retrieve,
        )
        self.update = to_raw_response_wrapper(
            rooms.update,
        )
        self.list = to_raw_response_wrapper(
            rooms.list,
        )
        self.delete = to_raw_response_wrapper(
            rooms.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._rooms.actions)

    @cached_property
    def sessions(self) -> SessionsResourceWithRawResponse:
        return SessionsResourceWithRawResponse(self._rooms.sessions)


class AsyncRoomsResourceWithRawResponse:
    def __init__(self, rooms: AsyncRoomsResource) -> None:
        self._rooms = rooms

        self.create = async_to_raw_response_wrapper(
            rooms.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            rooms.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            rooms.update,
        )
        self.list = async_to_raw_response_wrapper(
            rooms.list,
        )
        self.delete = async_to_raw_response_wrapper(
            rooms.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._rooms.actions)

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithRawResponse:
        return AsyncSessionsResourceWithRawResponse(self._rooms.sessions)


class RoomsResourceWithStreamingResponse:
    def __init__(self, rooms: RoomsResource) -> None:
        self._rooms = rooms

        self.create = to_streamed_response_wrapper(
            rooms.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            rooms.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            rooms.update,
        )
        self.list = to_streamed_response_wrapper(
            rooms.list,
        )
        self.delete = to_streamed_response_wrapper(
            rooms.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._rooms.actions)

    @cached_property
    def sessions(self) -> SessionsResourceWithStreamingResponse:
        return SessionsResourceWithStreamingResponse(self._rooms.sessions)


class AsyncRoomsResourceWithStreamingResponse:
    def __init__(self, rooms: AsyncRoomsResource) -> None:
        self._rooms = rooms

        self.create = async_to_streamed_response_wrapper(
            rooms.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            rooms.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            rooms.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rooms.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            rooms.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._rooms.actions)

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithStreamingResponse:
        return AsyncSessionsResourceWithStreamingResponse(self._rooms.sessions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["UserTagListParams", "Filter"]


class UserTagListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[starts_with]
    """


class Filter(TypedDict, total=False):
    starts_with: str
    """Filter tags by prefix"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["NumberHealthMetrics"]


class NumberHealthMetrics(BaseModel):
    inbound_outbound_ratio: float
    """The ratio of messages received to the number of messages sent."""

    message_count: int
    """The number of messages analyzed for the health metrics."""

    spam_ratio: float
    """The ratio of messages blocked for spam to the number of messages attempted."""

    success_ratio: float
    """
    The ratio of messages sucessfully delivered to the number of messages attempted.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import network_list_params, network_create_params, network_update_params, network_list_interfaces_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .default_gateway import (
    DefaultGatewayResource,
    AsyncDefaultGatewayResource,
    DefaultGatewayResourceWithRawResponse,
    AsyncDefaultGatewayResourceWithRawResponse,
    DefaultGatewayResourceWithStreamingResponse,
    AsyncDefaultGatewayResourceWithStreamingResponse,
)
from ...types.network_list_response import NetworkListResponse
from ...types.network_create_response import NetworkCreateResponse
from ...types.network_delete_response import NetworkDeleteResponse
from ...types.network_update_response import NetworkUpdateResponse
from ...types.network_retrieve_response import NetworkRetrieveResponse
from ...types.network_list_interfaces_response import NetworkListInterfacesResponse

__all__ = ["NetworksResource", "AsyncNetworksResource"]


class NetworksResource(SyncAPIResource):
    @cached_property
    def default_gateway(self) -> DefaultGatewayResource:
        return DefaultGatewayResource(self._client)

    @cached_property
    def with_raw_response(self) -> NetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NetworksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkCreateResponse:
        """
        Create a new Network.

        Args:
          name: A user specified name for the network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/networks",
            body=maybe_transform({"name": name}, network_create_params.NetworkCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkRetrieveResponse:
        """
        Retrieve a Network.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/networks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkUpdateResponse:
        """
        Update a Network.

        Args:
          name: A user specified name for the network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/networks/{id}",
            body=maybe_transform({"name": name}, network_update_params.NetworkUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkUpdateResponse,
        )

    def list(
        self,
        *,
        filter: network_list_params.Filter | Omit = omit,
        page: network_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkListResponse:
        """
        List all Networks.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/networks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    network_list_params.NetworkListParams,
                ),
            ),
            cast_to=NetworkListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkDeleteResponse:
        """
        Delete a Network.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/networks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkDeleteResponse,
        )

    def list_interfaces(
        self,
        id: str,
        *,
        filter: network_list_interfaces_params.Filter | Omit = omit,
        page: network_list_interfaces_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkListInterfacesResponse:
        """
        List all Interfaces for a Network.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name],
              filter[type], filter[status]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/networks/{id}/network_interfaces",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    network_list_interfaces_params.NetworkListInterfacesParams,
                ),
            ),
            cast_to=NetworkListInterfacesResponse,
        )


class AsyncNetworksResource(AsyncAPIResource):
    @cached_property
    def default_gateway(self) -> AsyncDefaultGatewayResource:
        return AsyncDefaultGatewayResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNetworksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkCreateResponse:
        """
        Create a new Network.

        Args:
          name: A user specified name for the network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/networks",
            body=await async_maybe_transform({"name": name}, network_create_params.NetworkCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkRetrieveResponse:
        """
        Retrieve a Network.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/networks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkUpdateResponse:
        """
        Update a Network.

        Args:
          name: A user specified name for the network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/networks/{id}",
            body=await async_maybe_transform({"name": name}, network_update_params.NetworkUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: network_list_params.Filter | Omit = omit,
        page: network_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkListResponse:
        """
        List all Networks.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/networks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    network_list_params.NetworkListParams,
                ),
            ),
            cast_to=NetworkListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkDeleteResponse:
        """
        Delete a Network.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/networks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NetworkDeleteResponse,
        )

    async def list_interfaces(
        self,
        id: str,
        *,
        filter: network_list_interfaces_params.Filter | Omit = omit,
        page: network_list_interfaces_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NetworkListInterfacesResponse:
        """
        List all Interfaces for a Network.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name],
              filter[type], filter[status]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/networks/{id}/network_interfaces",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    network_list_interfaces_params.NetworkListInterfacesParams,
                ),
            ),
            cast_to=NetworkListInterfacesResponse,
        )


class NetworksResourceWithRawResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

        self.create = to_raw_response_wrapper(
            networks.create,
        )
        self.retrieve = to_raw_response_wrapper(
            networks.retrieve,
        )
        self.update = to_raw_response_wrapper(
            networks.update,
        )
        self.list = to_raw_response_wrapper(
            networks.list,
        )
        self.delete = to_raw_response_wrapper(
            networks.delete,
        )
        self.list_interfaces = to_raw_response_wrapper(
            networks.list_interfaces,
        )

    @cached_property
    def default_gateway(self) -> DefaultGatewayResourceWithRawResponse:
        return DefaultGatewayResourceWithRawResponse(self._networks.default_gateway)


class AsyncNetworksResourceWithRawResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

        self.create = async_to_raw_response_wrapper(
            networks.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            networks.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            networks.update,
        )
        self.list = async_to_raw_response_wrapper(
            networks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            networks.delete,
        )
        self.list_interfaces = async_to_raw_response_wrapper(
            networks.list_interfaces,
        )

    @cached_property
    def default_gateway(self) -> AsyncDefaultGatewayResourceWithRawResponse:
        return AsyncDefaultGatewayResourceWithRawResponse(self._networks.default_gateway)


class NetworksResourceWithStreamingResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

        self.create = to_streamed_response_wrapper(
            networks.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            networks.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            networks.update,
        )
        self.list = to_streamed_response_wrapper(
            networks.list,
        )
        self.delete = to_streamed_response_wrapper(
            networks.delete,
        )
        self.list_interfaces = to_streamed_response_wrapper(
            networks.list_interfaces,
        )

    @cached_property
    def default_gateway(self) -> DefaultGatewayResourceWithStreamingResponse:
        return DefaultGatewayResourceWithStreamingResponse(self._networks.default_gateway)


class AsyncNetworksResourceWithStreamingResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

        self.create = async_to_streamed_response_wrapper(
            networks.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            networks.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            networks.update,
        )
        self.list = async_to_streamed_response_wrapper(
            networks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            networks.delete,
        )
        self.list_interfaces = async_to_streamed_response_wrapper(
            networks.list_interfaces,
        )

    @cached_property
    def default_gateway(self) -> AsyncDefaultGatewayResourceWithStreamingResponse:
        return AsyncDefaultGatewayResourceWithStreamingResponse(self._networks.default_gateway)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PartnerCampaignListParams"]


class PartnerCampaignListParams(TypedDict, total=False):
    page: int
    """The 1-indexed page number to get. The default value is `1`."""

    records_per_page: Annotated[int, PropertyInfo(alias="recordsPerPage")]
    """The amount of records per page, limited to between 1 and 500 inclusive.

    The default value is `10`.
    """

    sort: Literal[
        "assignedPhoneNumbersCount",
        "-assignedPhoneNumbersCount",
        "brandDisplayName",
        "-brandDisplayName",
        "tcrBrandId",
        "-tcrBranId",
        "tcrCampaignId",
        "-tcrCampaignId",
        "createdAt",
        "-createdAt",
        "campaignStatus",
        "-campaignStatus",
    ]
    """Specifies the sort order for results.

    If not given, results are sorted by createdAt in descending order.
    """
